# Generated by Django 3.0.4 on 2020-05-12 21:23

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_randomization", "0006_auto_20200213_2055"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="randomizationlist",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("site_name", "sid"),
                "permissions": (("display_assignment", "Can display assignment"),),
            },
        ),
    ]



from django.apps import apps as django_apps
from django.db import models
from django.db.models import UniqueConstraint

from edc_model.models import BaseUuidModel

from ..managers import CrfMetadataManager
from .crf_metadata_model_mixin import CrfMetadataModelMixin


class CrfMetadata(CrfMetadataModelMixin, BaseUuidModel):
    objects = CrfMetadataManager()

    def __str__(self) -> str:
        return (
            f"CrfMeta {self.model} {self.visit_schedule_name}.{self.schedule_name}."
            f"{self.visit_code}.{self.visit_code_sequence}@{self.timepoint} "
            f"{self.entry_status} {self.subject_identifier}"
        )

    def natural_key(self) -> tuple:
        return (
            self.model,
            self.subject_identifier,
            self.schedule_name,
            self.visit_schedule_name,
            self.visit_code,
            self.visit_code_sequence,
        )

    # noinspection PyTypeHints
    natural_key.dependencies = ("sites.Site",)

    @property
    def verbose_name(self) -> str:
        try:
            model = django_apps.get_model(self.model)
        except LookupError as e:
            return f"{e}. You need to regenerate metadata."
        return model._meta.verbose_name

    class Meta(CrfMetadataModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Crf collection status"
        verbose_name_plural = "Crf collection status"
        unique_together = ()
        constraints = (
            UniqueConstraint(
                fields=[
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                ],
                name="%(app_label)s_%(class)s_subject_iden_visit_uniq",
            ),
        )
        indexes = (
            *CrfMetadataModelMixin.Meta.indexes,
            *BaseUuidModel.Meta.indexes,
            models.Index(
                fields=[
                    "site",
                    "entry_status",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "subject_identifier",
                    "schedule_name",
                    "visit_schedule_name",
                ],
            ),
            models.Index(
                fields=[
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "visit_code",
                    "visit_code_sequence",
                    "model",
                    "entry_status",
                    "timepoint",
                    "show_order",
                ],
            ),
        )



# from https://gist.github.com/bruce-shi/fd0e3f5e2360c64bc9ce2efb254744f7
from collections import defaultdict

from django.db.models.signals import (
    post_delete,
    post_init,
    post_migrate,
    post_save,
    pre_delete,
    pre_init,
    pre_migrate,
    pre_save,
)


class DisableSignals:
    """
    Example usage:
    with DisableSignals():
        user.save()  # will not call any signals

    """

    def __init__(self, disabled_signals=None):
        self.stashed_signals = defaultdict(list)
        self.disabled_signals = disabled_signals or [
            pre_init,
            post_init,
            pre_save,
            post_save,
            pre_delete,
            post_delete,
            pre_migrate,
            post_migrate,
        ]

    def __enter__(self):
        for signal in self.disabled_signals:
            self.disconnect(signal)

    def __exit__(self, exc_type, exc_val, exc_tb):
        for signal in list(self.stashed_signals):
            self.reconnect(signal)

    def disconnect(self, signal):
        self.stashed_signals[signal] = signal.receivers
        signal.receivers = []

    def reconnect(self, signal):
        signal.receivers = self.stashed_signals.get(signal, [])
        del self.stashed_signals[signal]



# Generated by Django 3.0.9 on 2020-10-02 01:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_auth", "0018_auto_20191029_2039"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="export_format",
            field=models.CharField(
                blank=True,
                choices=[
                    ("CSV", "CSV (delimited by pipe `|`"),
                    (114, "Stata v10 or later"),
                    (117, "Stata v13 or later"),
                    (118, "Stata v14 or later"),
                    (119, "Stata v15 or later"),
                ],
                default="CSV",
                help_text="Note: requires export permissions",
                max_length=25,
                null=True,
                verbose_name="Export format",
            ),
        ),
    ]



from django.contrib import admin

from edc_model_admin.mixins import AddressModelAdminMixin

from ..admin_site import edc_lab_admin
from ..models import Shipper
from .base_model_admin import BaseModelAdmin


@admin.register(Shipper, site=edc_lab_admin)
class ShipperAdmin(BaseModelAdmin, AddressModelAdminMixin, admin.ModelAdmin):
    pass



# Generated by Django 2.1 on 2018-10-10 19:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_locator", "0014_auto_20181009_0545")]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="action_identifier",
            field=models.CharField(db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="parent_action_identifier",
            field=models.CharField(
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="related_action_identifier",
            field=models.CharField(
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="action_identifier",
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="parent_action_identifier",
            field=models.CharField(
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="related_action_identifier",
            field=models.CharField(
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
    ]



from __future__ import annotations

from collections import namedtuple
from typing import TYPE_CHECKING, Any

from django.core.exceptions import ValidationError

from edc_visit_schedule.visit import RequisitionCollection

from ...requisition import RequisitionMetadataUpdater
from ..rule_group import RuleGroup
from ..rule_group_meta_options import RuleGroupMetaOptions
from ..rule_group_metaclass import RuleGroupMetaclass

RuleResult = namedtuple("RuleResult", "target_panel entry_status")

if TYPE_CHECKING:
    from edc_visit_tracking.model_mixins import VisitModelMixin as Base

    from ...model_mixins.creates import CreatesMetadataModelMixin
    from ...models import RequisitionMetadata

    class RelatedVisitModel(CreatesMetadataModelMixin, Base):
        pass


class RequisitionRuleGroupMetaOptionsError(ValidationError):
    pass


class RequisitionRuleGroupMetaOptions(RuleGroupMetaOptions):
    def __init__(self, group_name: str, attrs: dict) -> None:
        super().__init__(group_name, attrs)
        self.requisition_model = self.options.get("requisition_model")
        if self.requisition_model:
            if len(self.requisition_model.split(".")) != 2:  # noqa: PLR2004
                self.requisition_model = f"{self.app_label}.{self.requisition_model}"
                self.options.update(requisition_model=self.requisition_model)
            self.options.update(target_models=[self.requisition_model])
            rules = {k: v for k, v in attrs.items() if not k.startswith("_")}
            if self.requisition_model == self.source_model:
                for name, rule in rules.items():
                    if not rule.source_panel:
                        raise RequisitionRuleGroupMetaOptionsError(
                            f'Rule expects "source_panel". Got '
                            f'requisition_model="{self.requisition_model}", '
                            f'source_model="{self.source_model}". '
                            f"See {group_name}.{name}.",
                            code="source_panel_expected",
                        )
            else:
                for name, rule in rules.items():
                    if rule.source_panel:
                        raise RequisitionRuleGroupMetaOptionsError(
                            f'Rule does not expect "source_panel". Got '
                            f'requisition_model="{self.requisition_model}", '
                            f'source_model="{self.source_model}". '
                            f"See {group_name}.{name}.",
                            code="source_panel_not_expected",
                        )

    @property
    def default_meta_options(self) -> list[str]:
        opts = super().default_meta_options
        opts.extend(["requisition_model"])
        return opts


class RequisitionMetaclass(RuleGroupMetaclass):
    rule_group_meta = RequisitionRuleGroupMetaOptions


class RequisitionRuleGroup(RuleGroup, metaclass=RequisitionMetaclass):
    metadata_updater_cls = RequisitionMetadataUpdater

    @classmethod
    def requisitions_for_visit(cls, visit=None) -> RequisitionCollection:
        """Returns a list of scheduled or unscheduled
        Requisitions depending on visit_code_sequence.
        """
        if visit.visit_code_sequence != 0:
            requisitions = RequisitionCollection(
                *visit.visit.requisitions_unscheduled,
                *visit.visit.requisitions_prn,
                name="requisitions_for_visit",
            )
        else:
            requisitions = RequisitionCollection(
                *visit.visit.requisitions,
                *visit.visit.requisitions_prn,
                name="requisitions_for_visit",
            )
        return requisitions

    @classmethod
    def evaluate_rules(
        cls: Any,
        related_visit: RelatedVisitModel = None,
        allow_create: bool | None = None,
    ) -> tuple[dict[str, dict[str, list[RuleResult]]], dict[str, RequisitionMetadata]]:
        """Returns a tuple of (rule_results, metadata_objects) where
        rule_results ...

        Metadata must exist.
        """
        rule_results = {}
        metadata_objects = {}
        for rule in cls._meta.options.get("rules"):
            rule_results[str(rule)] = {}
            if result := rule.run(related_visit=related_visit):
                for target_model, entry_status in result.items():
                    rule_results[str(rule)].update({target_model: []})
                    for target_panel in rule.target_panels:
                        # only do something if target_panel is in
                        # visit.requisitions
                        if target_panel.name in [
                            r.panel.name for r in cls.requisitions_for_visit(related_visit)
                        ]:
                            metadata_updater = cls.metadata_updater_cls(
                                related_visit=related_visit,
                                source_model=target_model,
                                source_panel=target_panel,
                                allow_create=allow_create,
                            )
                            metadata_obj = metadata_updater.get_and_update(
                                entry_status=entry_status
                            )
                            metadata_objects.update({target_panel: metadata_obj})
                            rule_results[str(rule)][target_model].append(
                                RuleResult(target_panel, entry_status)
                            )
        return rule_results, metadata_objects



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0018_alter_rxrefill_managers"),
    ]

    operations = [
        migrations.AddField(
            model_name="container",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="containertype",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="formulationtype",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="frequencyunits",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="route",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="units",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="unittype",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_facility", "0006_alter_healthfacility_health_facility_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthfacilitytypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-11 17:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_unblinding", "0002_auto_20210908_2318"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalunblindingrequest",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalunblindingreview",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="unblindingrequest",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="unblindingreview",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
    ]
